package cn.com.infosec.crypto.digests;

import cn.com.infosec.crypto.BlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.ExtendedDigest;
import cn.com.infosec.crypto.engines.GOST28147Engine;
import cn.com.infosec.crypto.params.KeyParameter;
import cn.com.infosec.crypto.params.ParametersWithSBox;

public class GOST3411Digest implements ExtendedDigest {
   private static final int DIGEST_LENGTH = 32;
   private static final byte[] C2 = new byte[]{0, -1, 0, -1, 0, -1, 0, -1, -1, 0, -1, 0, -1, 0, -1, 0, 0, -1, -1, 0, -1, 0, 0, -1, -1, 0, 0, 0, -1, -1, 0, -1};
   private byte[] H;
   private byte[] L;
   private byte[] M;
   private byte[] Sum;
   private byte[][] C;
   private byte[] xBuf;
   private int xBufOff;
   private long byteCount;
   private BlockCipher cipher;
   private byte[] K;
   byte[] a;
   short[] wS;
   short[] w_S;
   byte[] S;
   byte[] U;
   byte[] V;
   byte[] W;

   public GOST3411Digest() {
      GOST3411Digest var10000 = this;
      super();
      this.H = new byte[32];
      this.L = new byte[32];
      this.M = new byte[32];
      this.Sum = new byte[32];
      this.C = new byte[4][32];
      this.xBuf = new byte[32];
      GOST28147Engine var1;
      GOST28147Engine var10001 = var1 = new GOST28147Engine;
      var1.<init>();
      this.cipher = var1;
      this.K = new byte[32];
      this.a = new byte[8];
      this.wS = new short[16];
      this.w_S = new short[16];
      this.S = new byte[32];
      this.U = new byte[32];
      this.V = new byte[32];
      this.W = new byte[32];
      ParametersWithSBox var2;
      var2 = new ParametersWithSBox.<init>((CipherParameters)null, GOST28147Engine.getSBox("D-A"));
      var10001.init(true, var2);
      var10000.reset();
   }

   public GOST3411Digest(GOST3411Digest t) {
      GOST3411Digest var10000 = this;
      GOST3411Digest var10001 = t;
      GOST3411Digest var10002 = this;
      GOST3411Digest var10003 = t;
      GOST3411Digest var10004 = t;
      GOST3411Digest var10005 = t;
      GOST3411Digest var10006 = t;
      GOST3411Digest var10007 = t;
      GOST3411Digest var10008 = t;
      GOST3411Digest var10009 = t;
      GOST3411Digest var10010 = t;
      GOST3411Digest var10011 = t;
      super();
      this.H = new byte[32];
      this.L = new byte[32];
      this.M = new byte[32];
      this.Sum = new byte[32];
      this.C = new byte[4][32];
      this.xBuf = new byte[32];
      GOST28147Engine t1;
      GOST28147Engine var10013 = t1 = new GOST28147Engine;
      t1.<init>();
      this.cipher = t1;
      this.K = new byte[32];
      this.a = new byte[8];
      this.wS = new short[16];
      this.w_S = new short[16];
      this.S = new byte[32];
      this.U = new byte[32];
      this.V = new byte[32];
      this.W = new byte[32];
      ParametersWithSBox t2;
      t2 = new ParametersWithSBox.<init>((CipherParameters)null, GOST28147Engine.getSBox("D-A"));
      var10013.init(true, t2);
      this.reset();
      byte[] var19 = var10011.H;
      byte[] t3 = this.H;
      int var2 = var19.length;
      System.arraycopy(var19, 0, t3, 0, var2);
      byte[] var18 = var10010.L;
      t3 = this.L;
      var2 = var18.length;
      System.arraycopy(var18, 0, t3, 0, var2);
      byte[] var17 = var10009.M;
      t3 = this.M;
      var2 = var17.length;
      System.arraycopy(var17, 0, t3, 0, var2);
      byte[] var15 = var10008.Sum;
      t3 = this.Sum;
      var2 = var15.length;
      System.arraycopy(var15, 0, t3, 0, var2);
      byte[][] t4;
      byte[] var13 = (t4 = var10007.C)[1];
      byte[][] var16 = t4;
      t3 = this.C[1];
      var2 = var16[1].length;
      System.arraycopy(var13, 0, t3, 0, var2);
      byte[] var11 = (t4 = var10006.C)[2];
      byte[][] var14 = t4;
      t3 = this.C[2];
      var2 = var14[2].length;
      System.arraycopy(var11, 0, t3, 0, var2);
      byte[] var10 = (t4 = var10005.C)[3];
      byte[][] var12 = t4;
      t3 = this.C[3];
      var2 = var12[3].length;
      System.arraycopy(var10, 0, t3, 0, var2);
      byte[] var9 = var10004.xBuf;
      byte[] var3 = this.xBuf;
      int t5 = var9.length;
      System.arraycopy(var9, 0, var3, 0, t5);
      var10002.xBufOff = var10003.xBufOff;
      var10000.byteCount = var10001.byteCount;
   }

   private byte[] P(byte[] in) {
      for(int var2 = 0; var2 < 8; ++var2) {
         byte[] var3;
         byte[] var10000 = var3 = this.K;
         int var5 = var2 * 4;
         var3[var5] = in[var2];
         int var4 = var5 + 1;
         var3[var4] = in[var2 + 8];
         var4 = var5 + 2;
         var3[var4] = in[var2 + 16];
         var5 += 3;
         var10000[var5] = in[var2 + 24];
      }

      return this.K;
   }

   private byte[] A(byte[] in) {
      for(int var2 = 0; var2 < 8; ++var2) {
         this.a[var2] = (byte)(in[var2] ^ in[var2 + 8]);
      }

      System.arraycopy(in, 8, in, 0, 24);
      System.arraycopy(this.a, 0, in, 24, 8);
      return in;
   }

   private void E(byte[] key, byte[] s, int sOff, byte[] in, int inOff) {
      GOST3411Digest var10000 = this;
      BlockCipher var10001 = this.cipher;
      KeyParameter var6;
      var6 = new KeyParameter.<init>(key);
      var10001.init(true, var6);
      var10000.cipher.processBlock(in, inOff, s, sOff);
   }

   private void fw(byte[] in) {
      GOST3411Digest var10000 = this;
      GOST3411Digest var10001 = this;
      GOST3411Digest var10002 = this;
      this.cpyBytesToShort(in, this.wS);
      short[] var3 = this.w_S;
      short[] var2;
      short[] var4 = var2 = var10002.wS;
      var3[15] = (short)(var2[0] ^ var2[1] ^ var2[2] ^ var2[3] ^ var2[12] ^ var2[15]);
      System.arraycopy(var4, 1, var3, 0, 15);
      var10000.cpyShortToBytes(var10001.w_S, in);
   }

   private void finish() {
      long var1 = this.byteCount * 8L;
      byte[] var3 = this.L;
      this.LongToBytes(var1, var3, 0);

      while(this.xBufOff != 0) {
         this.update((byte)0);
      }

      this.processBlock(this.L, 0);
      this.processBlock(this.Sum, 0);
   }

   private void sumByteArray(byte[] in) {
      int var2 = 0;

      byte[] var4;
      for(int var3 = 0; var3 != (var4 = this.Sum).length; ++var3) {
         int var10000 = var2 += (var4[var3] & 255) + (in[var3] & 255);
         var4[var3] = (byte)var2;
         var2 = var10000 >>> 8;
      }

   }

   private void LongToBytes(long r, byte[] out, int outOff) {
      int var5 = outOff + 7;
      out[var5] = (byte)((int)(r >> 56));
      var5 = outOff + 6;
      out[var5] = (byte)((int)(r >> 48));
      var5 = outOff + 5;
      out[var5] = (byte)((int)(r >> 40));
      var5 = outOff + 4;
      out[var5] = (byte)((int)(r >> 32));
      var5 = outOff + 3;
      out[var5] = (byte)((int)(r >> 24));
      var5 = outOff + 2;
      out[var5] = (byte)((int)(r >> 16));
      var5 = outOff + 1;
      out[var5] = (byte)((int)(r >> 8));
      out[outOff] = (byte)((int)r);
   }

   private void cpyBytesToShort(byte[] S, short[] wS) {
      for(int var4 = 0; var4 < S.length / 2; ++var4) {
         int var3;
         wS[var4] = (short)(S[(var3 = var4 * 2) + 1] << 8 & '\uff00' | S[var3] & 255);
      }

   }

   private void cpyShortToBytes(short[] wS, byte[] S) {
      for(int var5 = 0; var5 < S.length / 2; ++var5) {
         int var3;
         int var4 = (var3 = var5 * 2) + 1;
         S[var4] = (byte)(wS[var5] >> 8);
         S[var3] = (byte)wS[var5];
      }

   }

   public String getAlgorithmName() {
      return "GOST3411";
   }

   public int getDigestSize() {
      return 32;
   }

   public void update(byte in) {
      byte[] var2 = this.xBuf;
      int var3;
      int var4;
      int var10000 = var4 = (var3 = this.xBufOff) + 1;
      this.xBufOff = var4;
      var2[var3] = in;
      if (var10000 == var2.length) {
         this.sumByteArray(var2);
         this.processBlock(this.xBuf, 0);
         this.xBufOff = 0;
      }

      ++this.byteCount;
   }

   public void update(byte[] in, int inOff, int len) {
      while(this.xBufOff != 0 && len > 0) {
         this.update(in[inOff]);
         ++inOff;
         --len;
      }

      byte[] var4;
      while(len > (var4 = this.xBuf).length) {
         int var10002 = len;
         int var10003 = inOff;
         int var10010 = inOff;
         inOff = var4.length;
         System.arraycopy(in, var10010, var4, 0, inOff);
         this.sumByteArray(this.xBuf);
         this.processBlock(this.xBuf, 0);
         byte[] inOff1;
         len = var10003 + (inOff1 = this.xBuf).length;
         int var6 = var10002 - inOff1.length;
         this.byteCount += (long)inOff1.length;
         inOff = len;
         len = var6;
      }

      while(len > 0) {
         this.update(in[inOff]);
         ++inOff;
         --len;
      }

   }

   protected void processBlock(byte[] in, int inOff) {
      System.arraycopy(in, inOff, this.M, 0, 32);
      System.arraycopy(this.H, 0, this.U, 0, 32);
      System.arraycopy(this.M, 0, this.V, 0, 32);

      int in1;
      for(in1 = 0; in1 < 32; ++in1) {
         this.W[in1] = (byte)(this.U[in1] ^ this.V[in1]);
      }

      in = this.P(this.W);
      byte[] inOff1 = this.S;
      byte[] var3 = this.H;
      this.E(in, inOff1, 0, var3, 0);
      byte in2 = 1;

      for(inOff = in2; inOff < 4; inOff = in1) {
         var3 = this.A(this.U);

         for(in1 = 0; in1 < 32; ++in1) {
            this.U[in1] = (byte)(var3[in1] ^ this.C[inOff][in1]);
         }

         this.V = this.A(this.A(this.V));

         for(in1 = 0; in1 < 32; ++in1) {
            this.W[in1] = (byte)(this.U[in1] ^ this.V[in1]);
         }

         in = this.P(this.W);
         var3 = this.S;
         int var4 = inOff * 8;
         byte[] var5 = this.H;
         this.E(in, var3, var4, var5, var4);
         in1 = inOff + 1;
      }

      for(in1 = 0; in1 < 12; ++in1) {
         this.fw(this.S);
      }

      for(in1 = 0; in1 < 32; ++in1) {
         byte[] var10000 = this.S;
         var10000[in1] ^= this.M[in1];
      }

      this.fw(this.S);

      for(in1 = 0; in1 < 32; ++in1) {
         (inOff1 = this.S)[in1] = (byte)(this.H[in1] ^ inOff1[in1]);
      }

      for(in1 = 0; in1 < 61; ++in1) {
         this.fw(this.S);
      }

      byte[] var6;
      in1 = (var6 = this.H).length;
      System.arraycopy(this.S, 0, var6, 0, in1);
   }

   public int doFinal(byte[] out, int outOff) {
      GOST3411Digest var10000 = this;
      this.finish();
      byte[] var10001 = this.H;
      int var3 = var10001.length;
      System.arraycopy(var10001, 0, out, outOff, var3);
      var10000.reset();
      return 32;
   }

   public void reset() {
      this.byteCount = 0L;
      this.xBufOff = 0;

      int var1;
      byte[] var2;
      for(var1 = 0; var1 < (var2 = this.H).length; ++var1) {
         var2[var1] = 0;
      }

      for(var1 = 0; var1 < (var2 = this.L).length; ++var1) {
         var2[var1] = 0;
      }

      for(var1 = 0; var1 < (var2 = this.M).length; ++var1) {
         var2[var1] = 0;
      }

      byte[][] var4;
      for(var1 = 0; var1 < (var4 = this.C)[1].length; ++var1) {
         var4[1][var1] = 0;
      }

      for(var1 = 0; var1 < (var4 = this.C)[3].length; ++var1) {
         var4[3][var1] = 0;
      }

      for(var1 = 0; var1 < (var2 = this.Sum).length; ++var1) {
         var2[var1] = 0;
      }

      for(var1 = 0; var1 < (var2 = this.xBuf).length; ++var1) {
         var2[var1] = 0;
      }

      byte[] var10000 = C2;
      byte[] var3 = this.C[2];
      var1 = var10000.length;
      System.arraycopy(var10000, 0, var3, 0, var1);
   }

   public int getByteLength() {
      return 32;
   }
}
